from ..cw_controller import CWController
# Class for /procurement/shipmentmethods
from . import shipment_method


class ShipmentMethodsAPI(CWController):
    def __init__(self):
        self.module_url = 'procurement'
        self.module = 'shipmentmethods'
        self._class = shipment_method.ShipmentMethod
        super().__init__()  # instance gets passed to parent object

    def get_shipment_methods(self):
        return super()._get()

    def create_shipment_method(self, a_shipment_method):
        return super()._create(a_shipment_method)

    def get_shipment_methods_count(self):
        return super()._get_count()

    def get_shipment_method_by_id(self, shipment_method_id):
        return super()._get_by_id(shipment_method_id)

    def delete_shipment_method_by_id(self, shipment_method_id):
        super()._delete_by_id(shipment_method_id)

    def replace_shipment_method(self, shipment_method_id):
        pass

    def update_shipment_method(self, shipment_method_id, key, value):
        return super()._update(shipment_method_id, key, value)


from cw_model import CWModel


class Opportunity(CWModel):

    def __init__(self, json_dict=None):
        self.id = None  # (Integer)
        self.name = None  # *(String(100))
        self.expectedCloseDate = None  # (String)
        self.type = None  # (OpportunityTypeReference)
        self.stage = None  # (OpportunityStageReference)
        self.status = None  # (OpportunityStatusReference)
        self.priority = None  # (OpportunityPriorityReference)
        self.notes = None  # (String)
        self.probability = None  # (OpportunityProbabilityReference)
        self.source = None  # (String(50))
        self.rating = None  # (OpportunityRatingReference)
        self.campaign = None  # (CampaignReference)
        self.primarySalesRep = None  # *(MemberReference)
        self.secondarySalesRep = None  # (MemberReference)
        self.locationId = None  # (Integer)
        self.businessUnitId = None  # (Integer)
        self.company = None  # *(CompanyReference)
        self.contact = None  # *(ContactReference)
        self.site = None  # *(SiteReference)
        self.customerPO = None  # (String(25))
        self.pipelineChangeDate = None  # (String)
        self.dateBecameLead = None  # (String)
        self.closedDate = None  # (String)
        self.closedBy = None  # (MemberReference)
        self.totalSalesTax = None  # (Number)
        self.shipToCompany = None  # (CompanyReference)
        self.shipToContact = None  # (ContactReference)
        self.shipToSite = None  # (SiteReference)
        self.billToCompany = None  # (CompanyReference)
        self.billToContact = None  # (ContactReference)
        self.billToSite = None  # (SiteReference)
        self.billingTerms = None  # (BillingTermsReference)
        self.taxCode = None  # (TaxCodeReference)
        self._info = None  # (Metadata)
        self.customFields = None  # (CustomFieldValue[])

        # initialize object with json dict
        super().__init__(json_dict)



from ..cw_model import CWModel


class AgreementAddition(CWModel):

    def __init__(self, json_dict=None):
        self.id = None  # (Integer)
        self.product = None  # *(IvItemReference)
        self.quantity = None  # (Number)
        self.lessIncluded = None  # (Number)
        self.unitPrice = None  # (Number)
        self.unitCost = None  # (Number)
        self.billCustomer = None  # *(Enum)
        self.effectiveDate = None  # (String)
        self.cancelledDate = None  # (String)
        self.taxableFlag = None  # (Boolean)
        self.serialNumber = None  # (String(50))
        self.invoiceDescription = None  # (String(6000))
        self.purchaseItemFlag = None  # (Boolean)
        self.specialOrderFlag = None  # (Boolean)
        self.agreementId = None  # (Integer)
        self.description = None  # (String)
        self.billedQuantity = None  # (Number)
        self.uom = None  # (String)
        self.extPrice = None  # (Number)
        self.extCost = None  # (Number)
        self.sequenceNumber = None  # (Number)
        self.margin = None  # (Number)
        self.prorateCost = None  # (Number)
        self.proratePrice = None  # (Number)
        self.extendedProrateCost = None  # (Number)
        self.extendedProratePrice = None  # (Number)
        self.prorateCurrentPeriodFlag = None  # (Boolean)
        self._info = None  # (Metadata)
        
        # initialize object with json dict
        super().__init__(json_dict)



from ..cw_model import CWModel


class Project(CWModel):

    def __init__(self, json_dict=None):
        self.id = None  # (Integer)
        self._info = None  # (Metadata)
        self.actualEnd = None  # (String)
        self.actualHours = None  # (Number)
        self.actualStart = None  # (String)
        self.agreement = None  # (AgreementReference)
        self.billExpenses = None  # (Enum)
        self.billingAmount = None  # (Number)
        self.billingAttention = None  # (String(50))
        self.billingMethod = None  # *(Enum)
        self.billingRateType = None  # (Enum)
        self.billingTerms = None  # (BillingTermsReference)
        self.billProducts = None  # (Enum)
        self.billProjectAfterClosedFlag = None  # (Boolean)
        self.billTime = None  # (Enum)
        self.billToCompany = None  # (CompanyReference)
        self.billToContact = None  # (ContactReference)
        self.billToSite = None  # (SiteReference)
        self.billUnapprovedTimeAndExpense = None  # (Boolean)
        self.board = None  # *(ProjectBoardReference)
        self.budgetAnalysis = None  # (Enum)
        self.budgetFlag = None  # (Boolean)
        self.budgetHours = None  # (Number)
        self.businessUnitId = None  # (Integer)
        self.company = None  # *(CompanyReference)
        self.contact = None  # (ContactReference)
        self.customerPO = None  # (String(50))
        self.description = None  # (String)
        self.downpayment = None  # (Number)
        self.estimatedEnd = None  # *(String)
        self.estimatedExpenseRevenue = None  # (Number)
        self.estimatedHours = None  # (Number)
        self.estimatedProductRevenue = None  # (Number)
        self.estimatedStart = None  # *(String)
        self.estimatedTimeRevenue = None  # (Number)
        self.expenseApprover = None  # (MemberReference)
        self.includeDependenciesFlag = None  # (Boolean)
        self.includeEstimatesFlag = None  # (Boolean)
        self.locationId = None  # (Integer)
        self.manager = None  # (MemberReference)
        self.name = None  # *(String(100))
        self.opportunity = None  # (OpportunityReference)
        self.projectTemplateId = None  # (Integer)
        self.restrictDownPaymentFlag = None  # (Boolean)
        self.scheduledEnd = None  # (String)
        self.scheduledHours = None  # (Number)
        self.scheduledStart = None  # (String)
        self.shipToCompany = None  # (CompanyReference)
        self.shipToContact = None  # (ContactReference)
        self.shipToSite = None  # (SiteReference)
        self.site = None  # (SiteReference)
        self.status = None  # (ProjectStatusReference)
        self.timeApprover = None  # (MemberReference)
        self.type = None  # (ProjectTypeReference)

        # initialize object with json dict
        super().__init__(json_dict)



from ..cw_model import CWModel


class Report(CWModel):

    def __init__(self, json_dict=None):
        self.column_definitions = None  # (JObject[])
        self.row_values = None  # (JObject[])

        # initialize object with json dict
        super().__init__(json_dict)



from ..cw_model import CWModel


class Product(CWModel):

    def __init__(self, json_dict=None):
        self.identifier = None  # (Enum)
        self.password = None  # (String)
        self.installedFlag = None  # (Boolean)
        self._info = None  # (Metadata)

        # initialize object with json dict
        super().__init__(json_dict)



from ..cw_controller import CWController
# Class for /ticket/tickets
from . import ticket


class TicketsAPI(CWController):
    def __init__(self):
        self.module_url = 'service'
        self.module = 'tickets'
        self._class = ticket.Ticket
        super().__init__()  # instance gets passed to parent object

    def get_tickets(self):
        return super()._get()

    def create_ticket(self, a_ticket):
        return super()._create(a_ticket)

    def get_tickets_count(self):
        return super()._get_count()

    def get_ticket_by_id(self, ticket_id):
        return super()._get_by_id(ticket_id)

    def delete_ticket_by_id(self, ticket_id):
        super()._delete_by_id(ticket_id)

    def replace_ticket(self, ticket_id):
        pass

    def update_ticket(self, ticket_id, key, value):
        return super()._update(ticket_id, key, value)

    def merge_ticket(self, a_ticket, target_ticket_id):
        # return super()._merge(a_ticket, target_ticket_id)
        pass

# get
# create
# count
# search
# getbyid
# delete
# replace
# update
# activities
# count
# timeentries
# count
# scheduleentries
# count
# documents
# count
# products
# count
# configurations
    def create_configurations(self, ticket_id, config_dict):
        # Create Configuration Association
        # /service/tickets/{id}/configurations
        # { id(Integer), deviceIdentifier(String) }
        the_id = '{}/configurations'.format(ticket_id)
        return super()._post_dict(the_id, config_dict)# count
# betbyid
# delete



from cw_model import CWModel


class Activity(CWModel):

    def __init__(self, json_dict=None):
        self.id = None  # (Integer)
        self.name = None  # *(String(100))
        self.type = None  # (ActivityTypeReference)
        self.company = None  # (CompanyReference)
        self.contact = None  # (ContactReference)
        self.phoneNumber = None  # (String(30))
        self.email = None  # (String(250))
        self.status = None  # (ActivityStatusReference)
        self.opportunity = None  # (OpportunityReference)
        self.ticket = None  # (TicketReference)
        self.agreement = None  # (AgreementReference)
        self.campaign = None  # (CampaignReference)
        self.notes = None  # (String)
        self.dateStart = None  # (String)
        self.dateEnd = None  # (String)
        self.assignedBy = None  # (MemberReference)
        self.assignTo = None  # *(MemberReference)
        self.scheduleStatus = None  # (ScheduleStatusReference)
        self.reminder = None  # (ReminderReference)
        self.where = None  # (ServiceLocationReference)
        self.notifyFlag = None  # (Boolean)
        self.mobileGuid = None  # (Guid)
        self._info = None  # (Metadata)
        self.customFields = None  # (CustomFieldValue[])

        # initialize object with json dict
        super().__init__(json_dict)



from ..cw_model import CWModel


class ProductPickingShippingDetail(CWModel):

    def __init__(self, json_dict=None):
        # With: id(Integer)
        # Search: ^(\s*)(\*?)([_a-z0-9]*)\s?(\(.*$)
        # Replace: $1self.$3 = None  # $2$4
        # To get: self.id = None  # (Integer)
        self.id = None  # (Integer)
        self.pickedQuantity = None  # *(Integer)
        self.shippedQuantity = None  # *(Integer)
        self.warehouse = None  # *(WarehouseReference)
        self.warehouseBin = None  # *(WarehouseBinReference)
        self.shipmentMethod = None  # (ShipmentMethodReference)
        self.serialNumber = None  # (String)
        self.serialNumberIds = None  # (Integer[])
        self.trackingNumber = None  # (String)
        self.productItem = None  # (ProductItemReference)
        self.lineNumber = None  # (Integer)
        self.quantity = None  # (Integer)
        self._info = None  # (Metadata)
        
        # initialize object with json dict
        super().__init__(json_dict)



from ..cw_controller import CWController
# Class for /system/members
from connectpyse.system import member


class MembersAPI(CWController):
    def __init__(self):
        self.module_url = 'system'
        self.module = 'members'
        self._class = member.Member
        super().__init__()  # instance gets passed to parent object

    def get_members(self):
        return super()._get()

    def create_member(self, a_type):
        return super()._create(a_type)

    def get_members_count(self):
        return super()._get_count()

    def get_member_by_id(self, type_id):
        return super()._get_by_id(type_id)

    def delete_member_by_id(self, type_id):
        super()._delete_by_id(type_id)

    def replace_member(self, type_id):
        pass

    def update_member(self, type_id, key, value):
        return super()._update(type_id, key, value)

    def merge_member(self, a_type, target_type_id):
        # return super()._merge(a_type, target_type_id)
        pass



from ..cw_model import CWModel


class ProductComponent(CWModel):

    def __init__(self, json_dict=None):
        # With: id(Integer)
        # Search: ^(\s*)(\*?)([_a-z0-9]*)\s?(\(.*$)
        # Replace: $1self.$3 = None  # $2$4
        # To get: self.id = None  # (Integer)
        self.id = None  # (Integer)
        self.sequenceNumber = None  # (Integer)
        self.quantity = None  # *(Number)
        self.catalogItem = None  # *(CatalogItemReference)
        self.hidePriceFlag = None  # (Boolean)
        self.hideItemIdentifierFlag = None  # (Boolean)
        self.hideDescriptionFlag = None  # (Boolean)
        self.hideQuantityFlag = None  # (Boolean)
        self.vendor = None  # (CompanyReference)
        self.parentProductItem = None  # (ProductItemReference)
        self.productItem = None  # (ProductItemReference)
        self.price = None  # (Number)
        self.cost = None  # (Number)
        self._info = None  # (Metadata)

        # initialize object with json dict
        super().__init__(json_dict)



from ..cw_model import CWModel


class PurchaseOrderLineItem(CWModel):

    def __init__(self, json_dict=None):
        # With: id(Integer)
        # Search: ^(\s*)(\*?)([_a-z0-9]*)\s?(\(.*$)
        # Replace: $1self.$3 = None  # $2$4
        # To get: self.id = None  # (Integer)
        self.id = None  # (Integer)
        self._info = None  # (Metadata)
        self.businessUnitId = None  # (Integer)
        self.cancelReason = None  # (String)
        self.closedFlag = None  # (Boolean)
        self.customerCity = None  # (String)
        self.customerCompany = None  # (CompanyReference)
        self.customerContact = None  # (ContactReference)
        self.customerCountry = None  # (CountryReference)
        self.customerExtension = None  # (String)
        self.customerName = None  # (String)
        self.customerPhone = None  # (String)
        self.customerSite = None  # (SiteReference)
        self.customerSiteName = None  # (String)
        self.customerState = None  # (String)
        self.customerStreetLine1 = None  # (String)
        self.customerStreetLine2 = None  # (String)
        self.customerZip = None  # (String)
        self.dateClosed = None  # (String)
        self.dropShipCustomerFlag = None  # (Boolean)
        self.enteredBy = None  # (String)
        self.freightCost = None  # (Number)
        self.freightPackingSlip = None  # (String)
        self.freightTaxTotal = None  # (Number)
        self.internalNotes = None  # (String)
        self.locationId = None  # *(Integer)
        self.poDate = None  # (String)
        self.poNumber = None  # (String(50))
        self.salesTax = None  # (Number)
        self.shipmentDate = None  # (String)
        self.shipmentMethod = None  # (ShipmentMethodReference)
        self.shippingInstructions = None  # (String)
        self.status = None  # *(PurchaseOrderStatusReference)
        self.subTotal = None  # (Number)
        self.taxCode = None  # (TaxCodeReference)
        self.taxFreightFlag = None  # (Boolean)
        self.taxPoFlag = None  # (Boolean)
        self.terms = None  # *(BillingTermsReference)
        self.total = None  # (Number)
        self.trackingNumber = None  # (String(50))
        self.updateShipmentInfo = None  # (Boolean)
        self.updateVendorOrderNumber = None  # (Boolean)
        self.vendorCompany = None  # *(CompanyReference)
        self.vendorContact = None  # (ContactReference)
        self.vendorInvoiceDate = None  # (String)
        self.vendorInvoiceNumber = None  # (String(50))
        self.vendorOrderNumber = None  # (String(50))
        self.vendorSite = None  # (SiteReference)
        self.warehouse = None  # (WarehouseReference)
        self.currency = None  # (CurrencyReference)

        # initialize object with json dict
        super().__init__(json_dict)



from cw_model import CWModel


class ScheduleEntry(CWModel):

    def __init__(self, json_dict=None):
        self.id = None  # (Integer)
        self.objectId = None  # (Integer)
        self.name = None  # (String(250))
        self.member = None  # *(MemberReference)
        self.where = None  # (ServiceLocationReference)
        self.dateStart = None  # (String)
        self.dateEnd = None  # (String)
        self.reminder = None  # (ReminderReference)
        self.status = None  # (ScheduleStatusReference)
        self.type = None  # *(ScheduleTypeReference)
        self.span = None  # (ScheduleSpanReference)
        self.doneFlag = None  # (Boolean)
        self.acknowledgedFlag = None  # (Boolean)
        self.ownerFlag = None  # (Boolean)
        self.allowScheduleConflictsFlag = None  # (Boolean)
        self.addMemberToProjectFlag = None  # (Boolean)
        self.projectRoleId = None  # (Integer)
        self.mobileGuid = None  # (Guid)
        self.closeDate = None  # (String)
        self.hours = None  # (Number)
        self._info = None  # (Metadata)

        # initialize object with json dict
        super().__init__(json_dict)



from ..cw_model import CWModel


class AgreementWorkRole(CWModel):

    def __init__(self, json_dict=None):
        self.id = None  # (Integer)
        self.workRole = None  # (WorkRoleReference)
        self.locationId = None  # (Integer)
        self.rateType = None  # *(Enum)
        self.rate = None  # (Number)
        self.limitTo = None  # (Number)
        self.effectiveDate = None  # (String)
        self.endingDate = None  # (String)
        self.agreementId = None  # (Integer)
        self._info = None  # (Metadata)

        # initialize object with json dict
        super().__init__(json_dict)



from ..cw_model import CWModel


class Document(CWModel):

    def __init__(self, json_dict=None):
        self.id = None  # (Integer)
        self.title = None  # (String)
        self.fileName = None  # (String)
        self.serverFileName = None  # (String)
        self.owner = None  # (String)
        self.linkFlag = None  # (Boolean)
        self.imageFlag = None  # (Boolean)
        self.publicFlag = None  # (Boolean)
        self.readOnlyFlag = None  # (Boolean)
        self._info = None  # (Metadata)

        # initialize object with json dict
        super().__init__(json_dict)



from ..cw_model import CWModel


class CatalogItem(CWModel):

    def __init__(self, json_dict=None):
        self.id = None  # (Integer)
        self.identifier = None  # *(String(30))
        self.description = None  # *(String(50))
        self.inactiveFlag = None  # (Boolean)
        self.subcategory = None  # *(ProductSubCategoryReference)
        self.type = None  # *(ProductTypeReference)
        self.productClass = None  # *(Enum)
        self.serializedFlag = None  # (Boolean)
        self.serializedCostFlag = None  # (Boolean)
        self.phaseProductFlag = None  # (Boolean)
        self.unitOfMeasure = None  # (UnitOfMeasureReference)
        self.minStockLevel = None  # (Integer)
        self.price = None  # (Number)
        self.cost = None  # (Number)
        self.priceAttribute = None  # (Enum)
        self.taxableFlag = None  # (Boolean)
        self.customerDescription = None  # *(String(6000))
        self.manufacturer = None  # (ManufacturerReference)
        self.manufacturerPartNumber = None  # (String(50))
        self.vendor = None  # (CompanyReference)
        self.vendorSku = None  # (String(50))
        self.notes = None  # (String)
        self.integrationXRef = None  # (String(50))
        self.dateEntered = None  # (String)
        self.category = None  # (ProductCategoryReference)
        self._info = None  # (Metadata)

        # initialize object with json dict
        super().__init__(json_dict)



from ..cw_model import CWModel


class AccountingUnpostedExpense(CWModel):

    def __init__(self, json_dict=None):
        self.id = None  # (Integer)
        self.locationId = None  # (Integer)
        self.departmentId = None  # (Integer)
        self.company = None  # (CompanyReference)
        self.accountNumber = None  # (String)
        self.creditAccount = None  # (String)
        self.expenseDetailId = None  # (Integer)
        self.expenseType = None  # (ExpenseTypeReference)
        self.classification = None  # (Enum)
        self.glType = None  # (Enum)
        self.member = None  # (MemberReference)
        self.dateExpense = None  # (String)
        self.chargeCode = None  # (ChargeCodeReference)
        self.chargeDescription = None  # (String)
        self.inPolicy = None  # (Boolean)
        self.paymentMethod = None  # (PaymentMethodReference)
        self.currency = None  # (CurrencyReference)
        self.total = None  # (Number)
        self.billableAmount = None  # (Number)
        self.nonBillableAmount = None  # (Number)
        self.agreement = None  # (AgreementReference)
        self.agreementAmountCovered = None  # (Number)
        self.ticket = None  # (TicketReference)
        self.project = None  # (ProjectReference)
        self.projectPhase = None  # (ProjectPhaseReference)
        self.taxCode = None  # (TaxCodeReference)
        self.avalaraTaxFlag = None  # (Boolean)
        self.itemTaxableFlag = None  # (Boolean)
        self.salesTaxAmount = None  # (Number)
        self.stateTaxFlag = None  # (Boolean)
        self.stateTaxXref = None  # (String)
        self.stateTaxAmount = None  # (Number)
        self.countyTaxFlag = None  # (Boolean)
        self.countyTaxXref = None  # (String)
        self.countyTaxAmount = None  # (Number)
        self.cityTaxFlag = None  # (Boolean)
        self.cityTaxXref = None  # (String)
        self.cityTaxAmount = None  # (Number)
        self.countryTaxFlag = None  # (Boolean)
        self.countryTaxXref = None  # (String)
        self.countryTaxAmount = None  # (Number)
        self.compositeTaxFlag = None  # (Boolean)
        self.compositeTaxXref = None  # (String)
        self.compositeTaxAmount = None  # (Number)
        self.dateClosed = None  # (String)
        self._info = None  # (Metadata)

        # initialize object with json dict
        super().__init__(json_dict)



from ..cw_model import CWModel


class TicketStopwatch(CWModel):

    def __init__(self, json_dict=None):
        self._info = None  # (Metadata)
        self.agreement = None  # (AgreementReference)
        self.billableOption = None  # (Enum)
        self.businessUnitId = None  # (Integer)
        self.dateEntered = None  # (String)
        self.endTime = None  # (String)
        self.id = None  # (Integer)
        self.internalNotes = None  # (String)
        self.locationId = None  # (Integer)
        self.member = None  # *(MemberReference)
        self.mobileGuid = None  # (Guid)
        self.notes = None  # (String(4000))
        self.serviceStatus = None  # (ServiceStatusReference)
        self.startTime = None  # (String)
        self.status = None  # *(Enum)
        self.ticket = None  # *(TicketReference)
        self.ticketMobileGuid = None  # (Guid)
        self.totalPauseTime = None  # (Integer)
        self.workRole = None  # (WorkRoleReference)
        self.workType = None  # (WorkTypeReference)
        self.showNotesInDiscussionFlag = None  # (Boolean)
        self.showNotesInInternalFlag = None  # (Boolean)
        self.showNotesInResolutionFlag = None  # (Boolean)
        self.emailNotesToContactFlag = None  # (Boolean)
        self.emailNotesToResourcesFlag = None  # (Boolean)

        # initialize object with json dict
        super().__init__(json_dict)



from ..cw_model import CWModel


class ScheduleStopwatch(CWModel):

    def __init__(self, json_dict=None):
        self._info = None  # (Metadata)
        self.agreement = None  # (AgreementReference)
        self.billableOption = None  # (Enum)
        self.businessUnitId = None  # (Integer)
        self.dateEntered = None  # (String)
        self.endTime = None  # (String)
        self.id = None  # (Integer)
        self.internalNotes = None  # (String)
        self.locationId = None  # (Integer)
        self.member = None  # *(MemberReference)
        self.mobileGuid = None  # (Guid)
        self.notes = None  # (String(4000))
        self.scheduleId = None  # *(Integer)
        self.scheduleMobileGuid = None  # (Guid)
        self.startTime = None  # (String)
        self.status = None  # *(Enum)
        self.totalPauseTime = None  # (Integer)
        self.workRole = None  # (WorkRoleReference)
        self.workType = None  # (WorkTypeReference)

        # initialize object with json dict
        super().__init__(json_dict)



from ..cw_model import CWModel


class ConnectwiseHostedSetup(CWModel):

    def __init__(self, json_dict=None):
        self.id = None  # (Integer)
        self.screenId = None  # *(Integer)
        self.description = None  # *(String(255))
        self.url = None  # *(String(1024))
        self.type = None  # *(Enum)
        self.origin = None  # (String(50))
        self.podHeight = None  # (Integer)
        self.toolbarButtonDialogHeight = None  # (Integer)
        self.toolbarButtonDialogWidth = None  # (Integer)
        self.toolbarButtonText = None  # *(String(50))
        self.toolbarButtonToolTip = None  # (String(50))
        self.toolbarButtonIconDocumentId = None  # (Integer)
        self.disabledFlag = None  # (Boolean)
        self.createdBy = None  # (String)
        self.dateCreated = None  # (String)
        self._info = None  # (Metadata)

        # initialize object with json dict
        super().__init__(json_dict)



from ..cw_model import CWModel


class TimeEntry(CWModel):

    def __init__(self, json_dict=None):
        self.id = None  # (Integer)
        self.company = None  # *(CompanyReference)
        self.chargeToId = None  # *(Integer)
        self.chargeToType = None  # *(Enum)
        self.member = None  # (MemberReference)
        self.locationId = None  # (Integer)
        self.businessUnitId = None  # (Integer)
        self.workType = None  # (WorkTypeReference)
        self.workRole = None  # (WorkRoleReference)
        self.agreement = None  # (AgreementReference)
        self.timeStart = None  # *(String)
        self.timeEnd = None  # (String)
        self.hoursDeduct = None  # (Number)
        self.actualHours = None  # (Number)
        self.billableOption = None  # (Enum)
        self.notes = None  # (String)
        self.internalNotes = None  # (String)
        self.addToDetailDescriptionFlag = None  # (Boolean)
        self.addToInternalAnalysisFlag = None  # (Boolean)
        self.addToResolutionFlag = None  # (Boolean)
        self.emailResourceFlag = None  # (Boolean)
        self.emailContactFlag = None  # (Boolean)
        self.emailCcFlag = None  # (Boolean)
        self.emailCc = None  # (String)
        self.hoursBilled = None  # (Number)
        self.enteredBy = None  # (String)
        self.dateEntered = None  # (String)
        self.invoice = None  # (InvoiceReference)
        self.hourlyRate = None  # (Number)
        self._info = None  # (Metadata)
        self.customFields = None  # (CustomFieldValue[])

        # initialize object with json dict
        super().__init__(json_dict)



from cw_model import CWModel


class Member(CWModel):

    def __init__(self, json_dict=None):
        self.adminFlag = None  # (Boolean)
        self.allowExpensesEnteredAgainstCompaniesFlag = None  # (Boolean)
        self.allowInCellEntryOnTimeSheet = None  # (Boolean)
        self.billableForecast = None  # (Number)
        self.calendar = None  # (CalendarReference)
        self.calendarSyncIntegrationFlag = None  # (Boolean)
        self.country = None  # (CountryReference)
        self.dailyCapacity = None  # (Number)
        self.daysTolerance = None  # (Integer)
        self.defaultDepartmentId = None  # *(Integer)
        self.defaultEmail = None  # *(Enum)
        self.defaultLocationId = None  # *(Integer)
        self.defaultPhone = None  # *(Enum)
        self.disableOnlineFlag = None  # (Boolean)
        self.enableLdapAuthenticationFlag = None  # (Boolean)
        self.enableMobileFlag = None  # (Boolean)
        self.enableMobileGpsFlag = None  # (Boolean)
        self.enterTimeAgainstCompanyFlag = None  # (Boolean)
        self.expenseApprover = None  # *(MemberReference)
        self.firstName = None  # *(String(30))
        self.hideMemberInDispatchPortalFlag = None  # (Boolean)
        self.hireDate = None  # *(String)
        self.homeEmail = None  # (String(250))
        self.homeExtension = None  # (String(10))
        self.homePhone = None  # (String(15))
        self.hourlyCost = None  # (Number)
        self.hourlyRate = None  # (Number)
        self.id = None  # (Integer)
        self.identifier = None  # *(String(15))
        self.inactiveDate = None  # (String)
        self.inactiveFlag = None  # (Boolean)
        self.includeInUtilizationReportingFlag = None  # (Boolean)
        self.lastLogin = None  # (String)
        self.lastName = None  # *(String(30))
        self.licenseClass = None  # *(Enum)
        self.mapiName = None  # (String)
        self.middleInitial = None  # (String)
        self.minimumHours = None  # (Number)
        self.mobileEmail = None  # (String(250))
        self.mobileExtension = None  # (String(10))
        self.mobilePhone = None  # (String(15))
        self.notes = None  # (String)
        self.officeEmail = None  # (String(250))
        self.officeExtension = None  # (String(10))
        self.officePhone = None  # (String(15))
        self.projectDefaultBoard = None  # (ProjectBoardReference)
        self.projectDefaultDepartmentId = None  # (Integer)
        self.projectDefaultLocationId = None  # (Integer)
        self.reportsTo = None  # (MemberReference)
        self.requireExpenseEntryFlag = None  # (Boolean)
        self.requireStartAndEndTimeOnTimeEntryFlag = None  # (Boolean)
        self.requireTimeSheetEntryFlag = None  # (Boolean)
        self.restrictDefaultSalesTerritoryFlag = None  # (Boolean)
        self.restrictDefaultWarehouseBinFlag = None  # (Boolean)
        self.restrictDefaultWarehouseFlag = None  # (Boolean)
        self.restrictDepartmentFlag = None  # (Boolean)
        self.restrictLocationFlag = None  # (Boolean)
        self.restrictProjectDefaultDepartmentFlag = None  # (Boolean)
        self.restrictProjectDefaultLocationFlag = None  # (Boolean)
        self.restrictScheduleFlag = None  # (Boolean)
        self.restrictServiceDefaultDepartmentFlag = None  # (Boolean)
        self.restrictServiceDefaultLocationFlag = None  # (Boolean)
        self.salesDefaultLocationId = None  # *(Integer)
        self.scheduleCapacity = None  # (Number)
        self.scheduleDefaultDepartmentId = None  # (Integer)
        self.scheduleDefaultLocationId = None  # (Integer)
        self.securityLevel = None  # (String)
        self.securityLocationId = None  # (Integer)
        self.securityRole = None  # (SecurityRoleReference)
        self.serviceDefaultBoard = None  # (BoardReference)
        self.serviceDefaultDepartmentId = None  # (Integer)
        self.serviceDefaultLocationId = None  # (Integer)
        self.serviceLocation = None  # (ServiceLocationReference)
        self.serviceTeams = None  # (Array)
        self.timeApprover = None  # *(MemberReference)
        self.timeReminderEmailFlag = None  # (Boolean)
        self.timeSheetStartDate = None  # (String)
        self.timeZone = None  # *(TimeZoneReference)
        self.title = None  # (String(50))
        self.type = None  # (MemberTypeReference)
        self.vendorNumber = None  # (String)
        self.warehouse = None  # (WarehouseReference)
        self.warehouseBin = None  # (WarehouseBinReference)
        self.workRole = None  # *(WorkRoleReference)
        self.workType = None  # (WorkTypeReference)
        self._info = None  # (Metadata)
        self.System_Flag = None  # (Boolean)

        # initialize object with json dict
        super().__init__(json_dict)



from ..cw_model import CWModel


class AdjustmentDetail(CWModel):

    def __init__(self, json_dict=None):
        self.id = None  # (Integer)
        self.catalogItem = None  # *(CatalogItemReference)
        self.description = None  # (String(50))
        self.quantityOnHand = None  # (Number)
        self.unitCost = None  # (Number)
        self.warehouse = None  # *(WarehouseReference)
        self.warehouseBin = None  # *(WarehouseBinReference)
        self.quantityAdjusted = None  # *(Integer)
        self.serialNumber = None  # (String(1000))
        self.adjustment = None  # (AdjustmentReference)
        self._info = None  # (Metadata)

        # initialize object with json dict
        super().__init__(json_dict)



from ..cw_model import CWModel


class AccountingBatch(CWModel):

    def __init__(self, json_dict=None):
        self.thruDate = None  # (String)
        self.transactionsClosedDate = None  # (String)
        self.locationId = None  # (Integer)
        self.summarizeInvoices = None  # (Integer)
        self.includedInvoiceIds = None  # (Integer[])
        self.includedExpenseIds = None  # (Integer[])
        self.includedProductIds = None  # (Integer[])
        self.excludedInvoiceIds = None  # (Integer[])
        self.excludedExpenseIds = None  # (Integer[])
        self.excludedProductIds = None  # (Integer[])
        self.id = None  # (Integer)
        self.batchIdentifier = None  # *(String(50))
        self.exportInvoicesFlag = None  # (Boolean)
        self.exportExpensesFlag = None  # (Boolean)
        self.exportProductsFlag = None  # (Boolean)
        self.closedFlag = None  # (Boolean)
        self._info = None  # (Metadata)

        # initialize object with json dict
        super().__init__(json_dict)



from ..cw_model import CWModel


class ProjectPhase(CWModel):

    def __init__(self, json_dict=None):
        self.id = None  # (Integer)
        self.projectId = None  # (Integer)
        self.description = None  # *(String(100))
        self.board = None  # (ProjectBoardReference)
        self.status = None  # (PhaseStatusReference)
        self.agreement = None  # (AgreementReference)
        self.opportunity = None  # (OpportunityReference)
        self.parentPhase = None  # (ProjectPhaseReference)
        self.wbsCode = None  # (String(50))
        self.billTime = None  # (Enum)
        self.billExpenses = None  # (Enum)
        self.billProducts = None  # (Enum)
        self.markAsMilestoneFlag = None  # (Boolean)
        self.notes = None  # (String)
        self.deadlineDate = None  # (String)
        self.billSeparatelyFlag = None  # (Boolean)
        self.billingMethod = None  # *(Enum)
        self.scheduledHours = None  # (Number)
        self.scheduledStart = None  # (String)
        self.scheduledEnd = None  # (String)
        self.actualHours = None  # (Number)
        self.actualStart = None  # (String)
        self.actualEnd = None  # (String)
        self.budgetHours = None  # (Number)
        self.locationId = None  # (Integer)
        self.businessUnitId = None  # (Integer)
        self.hourlyRate = None  # (Number)
        self.billingStartDate = None  # (String)
        self.billPhaseClosedFlag = None  # (Boolean)
        self.billProjectClosedFlag = None  # (Boolean)
        self.downpayment = None  # (Number)
        self.poNumber = None  # (String(25))
        self.poAmount = None  # (Number)
        self.estimatedTimeCost = None  # (Number)
        self.estimatedExpenseCost = None  # (Number)
        self.estimatedProductCost = None  # (Number)
        self.estimatedTimeRevenue = None  # (Number)
        self.estimatedExpenseRevenue = None  # (Number)
        self.estimatedProductRevenue = None  # (Number)
        self._info = None  # (Metadata)

        # initialize object with json dict
        super().__init__(json_dict)



from ..cw_model import CWModel


class AuditTrail(CWModel):

    def __init__(self, json_dict=None):
        self.text = None  # (String)
        self.enteredDate = None  # (String)
        self.enteredBy = None  # (String)
        self.auditType = None  # (String)
        self.auditSubType = None  # (String)
        self.auditSource = None  # (String)

        # initialize object with json dict
        super().__init__(json_dict)



# Parent class for module controller classes
from .config import API_URL, basic_auth
from .restapi import Client


class CWController(Client):
    def __init__(self):
        # self.module_url comes from child
        # self.module comes from child
        # self._class comes from child
        self.conditions = ''
        self.orderBy = ''
        self.childconditions = ''
        self.customfieldconditions = ''
        self.page = ''
        self.pageSize = ''
        super().__init__('{}/{}'.format(API_URL, self.module_url))

    def _format_user_params(self):
        user_params = {}
        for param in ['conditions', 'orderBy', 'childconditions', 'customfieldconditions', 'page', 'pageSize']:
            user_params[param] = getattr(self, param)
        return user_params

    def _get(self):
        json_results = getattr(self, self.module).get(user_headers=basic_auth,
                                                      user_params=self._format_user_params())
        for json in json_results:
            yield self._class(json)

    def _create(self, a_object):
        # Ideally take the_item and submit that as the user_data
        try:
            clean_dict = {k: v for k, v in a_object.__dict__.items() if v}
        except Exception as e:
            print(repr(e))
            return False
        an_instance = self._class(
            getattr(self, self.module).post(user_data=clean_dict, user_headers=basic_auth))
        return an_instance

    def _get_count(self):
        json_results = getattr(self, self.module).get(the_id='count', user_headers=basic_auth,
                                                      user_params=self._format_user_params())
        count = json_results['count']
        return count

    def _get_by_id(self, item_id):
        an_instance = self._class(getattr(self, self.module).get(the_id=item_id, user_headers=basic_auth))
        return an_instance

    def _delete_by_id(self, item_id):
        return getattr(self, self.module).delete(the_id=item_id, user_headers=basic_auth)

    def _replace(self, item_id):  # TODO: test
        an_instance = self._class(
            getattr(self, self.module).put(the_id=item_id, user_data=self.clean_dict, user_headers=basic_auth))
        return an_instance

    def _update(self, item_id, key, value):
        # build PatchOperation dict
        patch_operation = [{
            'op': 'replace',
            'path': key,
            'value': value
        }]
        # call Patch method on API
        an_instance = self._class(getattr(self, self.module).patch(the_id=item_id, user_data=patch_operation,
                                                                   user_headers=basic_auth))
        return an_instance

    def _merge(self, a_object, target_id):  # TODO: test
        # try:
        #     clean_dict = {k: v for k, v in a_object.__dict__.items() if v}
        # except Exception as e:
        #     print(repr(e))
        #     return False
        # clean_dict['toCompanyId'] = target_id
        # response = getattr(self, self.module).post(user_data=clean_dict, the_id='/'.join([str(a_object.id), 'merge']),
        #                                            user_headers=basic_auth)
        # return response
        pass

    def _post_dict(self, item_id, the_dict):  # TODO: test
        # Ideally take the_item and submit that as the user_data
        try:
            clean_dict = {k: v for k, v in the_dict.items() if v}
        except Exception as e:
            print(repr(e))
            return False
        an_instance = self._class(
            getattr(self, self.module).post(the_id=item_id, user_data=clean_dict, user_headers=basic_auth))
        return an_instance




from ..cw_model import CWModel


class Board(CWModel):

    def __init__(self, json_dict=None):
        self.id = None  # (Integer)
        self.name = None  # *(String(50))
        self.locationId = None  # *(Integer)
        self.businessUnitId = None  # *(Integer)
        self.inactive = None  # (Boolean)
        self.signOffTemplate = None  # (ServiceSignoffReference)
        self.sendToContact = None  # (Boolean)
        self.contactTemplateId = None  # (Integer)
        self.sendToResource = None  # (Boolean)
        self.resourceTemplateId = None  # (Integer)
        self.projectFlag = None  # (Boolean)
        self.showDependenciesFlag = None  # (Boolean)
        self.showEstimatesFlag = None  # (Boolean)
        self._info = None  # (Metadata)

        # initialize object with json dict
        super().__init__(json_dict)



from ..cw_model import CWModel


class ClassName(CWModel):

    def __init__(self, json_dict=None):
        # With: id(Integer)
        # Search: ^(\s*)(\*?)([_a-z0-9]*)\s?(\(.*$)
        # Replace: $1self.$3 = None  # $2$4
        # To get: self.id = None  # (Integer)


        # initialize object with json dict
        super().__init__(json_dict)



from ..cw_model import CWModel


class OpportunityForecast(CWModel):

    def __init__(self, json_dict=None):
        self.id = None  # (Integer)
        self.name = None  # (String(50))
        self.revenue = None  # (Number)
        self.cost = None  # (Number)
        self.type = None  # *(Enum)
        self.status = None  # (OpportunityStatusReference)
        self.includedFlag = None  # (Boolean)
        self.recurring = None  # (ProductRecurring)
        self.percent = None  # (Number)
        self.margin = None  # (Number)
        self.opportunityId = None  # (Integer)
        self._info = None  # (Metadata)

        # initialize object with json dict
        super().__init__(json_dict)



from ..cw_controller import CWController
# Class for /system/callbacks
from connectpyse.system import callback


class CallbacksAPI(CWController):
    def __init__(self):
        self.module_url = 'system'
        self.module = 'callbacks'
        self._class = callback.Callback
        super().__init__()  # instance gets passed to parent object

    def get_callbacks(self):
        return super()._get()

    def create_callback(self, a_entry):
        return super()._create(a_entry)

    def get_callbacks_count(self):
        return super()._get_count()

    def get_callback_by_id(self, entry_id):
        return super()._get_by_id(entry_id)

    def delete_callback_by_id(self, entry_id):
        super()._delete_by_id(entry_id)

    def replace_callback(self, entry_id):
        pass

    def update_callback(self, entry_id, key, value):
        return super()._update(entry_id, key, value)

    def merge_callback(self, a_entry, target_entry_id):
        # return super()._merge(a_callback, target_callback_id)
        pass



from ..cw_model import CWModel


class CampaignFormSubmitted(CWModel):

    def __init__(self, json_dict=None):
        self.id = None  # (Integer)
        self.campaignId = None  # (Integer)
        self.contactId = None  # *(Integer)
        self.dateSubmitted = None  # (String)
        self.url = None  # *(String(2083))
        self.queryString = None  # (String)
        self.pageType = None  # (String)
        self.pageSubType = None  # (String)
        self.topic = None  # (String)
        self.version = None  # (String)
        self.status = None  # (String)

        # initialize object with json dict
        super().__init__(json_dict)



from ..cw_model import CWModel


class Adjustment(CWModel):

    def __init__(self, json_dict=None):
        self.id = None  # (Integer)
        self.identifier = None  # *(String(50))
        self.type = None  # *(AdjustmentTypeReference)
        self.reason = None  # (String(100))
        self.notes = None  # (String)
        self.closedFlag = None  # (Boolean)
        self.closedBy = None  # (String)
        self.closedDate = None  # (String)
        self._info = None  # (Metadata)

        # initialize object with json dict
        super().__init__(json_dict)



from ..cw_model import CWModel


class UserDefinedField(CWModel):

    def __init__(self, json_dict=None):
        self.id = None  # (Integer)
        self.podId = None  # *(Integer)
        self.caption = None  # *(String(25))
        self.sequenceNumber = None  # *(Integer)
        self.helpText = None  # (String(1000))
        self.fieldTypeIdentifier = None  # *(Enum)
        self.numberDecimals = None  # (Integer)
        self.entryTypeIdentifier = None  # (Enum)
        self.requiredFlag = None  # (Boolean)
        self.displayOnScreenFlag = None  # (Boolean)
        self.readOnlyFlag = None  # (Boolean)
        self.listViewFlag = None  # (Boolean)
        self.buttonUrl = None  # (String(1000))
        self.options = None  # (UserDefinedFieldOption[])
        self.businessUnitIds = None  # (Integer[])
        self.locationIds = None  # (Integer[])
        self.addAllBusinessUnits = None  # (Boolean)
        self.removeAllBusinessUnits = None  # (Boolean)
        self.addAllLocations = None  # (Boolean)
        self.removeAllLocations = None  # (Boolean)
        self.dateCreated = None  # (String)
        self._info = None  # (Metadata)

        # initialize object with json dict
        super().__init__(json_dict)




class CWModel(object):
    def __init__(self, json_dict=None):
        if json_dict is not None:
            self.__dict__.update(json_dict)

    def __repr__(self):
        string = ''
        for k, v in self.__dict__.items():
            string = ''.join([string, '{}: {}\n'.format(k, v)])
        return string



from ..cw_model import CWModel


class Invoice(CWModel):

    def __init__(self, json_dict=None):
        self.id = None  # (Integer)
        self.invoiceNumber = None  # (String(15))
        self.type = None  # *(Enum)
        self.status = None  # (BillingStatusReference)
        self.company = None  # *(CompanyReference)
        self.billToCompany = None  # (CompanyReference)
        self.shipToCompany = None  # (CompanyReference)
        self.applyToType = None  # (Enum)
        self.applyToId = None  # (Integer)
        self.attention = None  # (String(60))
        self.billingSite = None  # (SiteReference)
        self.shippingSite = None  # (SiteReference)
        self.billingTerms = None  # (BillingTermsReference)
        self.reference = None  # (String(50))
        self.customerPO = None  # (String(50))
        self.templateSetupId = None  # (Integer)
        self.emailTemplateId = None  # (Integer)
        self.addToBatchEmailList = None  # (Boolean)
        self.date = None  # (String)
        self.restrictDownpaymentFlag = None  # (Boolean)
        self.locationId = None  # (Integer)
        self.departmentId = None  # (Integer)
        self.territoryId = None  # (Integer)
        self.topComment = None  # (String)
        self.bottomComment = None  # (String)
        self.taxableFlag = None  # (Boolean)
        self.taxCode = None  # (TaxCodeReference)
        self.internalNotes = None  # (String)
        self.downpaymentPreviouslyTaxedFlag = None  # (Boolean)
        self.serviceTotal = None  # (Number)
        self.dueDate = None  # (String)
        self.expenseTotal = None  # (Number)
        self.productTotal = None  # (Number)
        self.previousProgressApplied = None  # (Number)
        self.serviceAdjustmentAmount = None  # (Number)
        self.agreementAmount = None  # (Number)
        self.downpaymentApplied = None  # (Number)
        self.subtotal = None  # (Number)
        self.total = None  # (Number)
        self.remainingDownpayment = None  # (Number)
        self.salesTax = None  # (Number)
        self.adjustmentReason = None  # (String)
        self.adjustedBy = None  # (String)
        self.payments = None  # (Number)
        self.credits = None  # (Number)
        self.balance = None  # (Number)
        self.specialInvoiceFlag = None  # (Boolean)

        # initialize object with json dict
        super().__init__(json_dict)



from setuptools import setup


def readme():
    with open('README.md') as f:
        return f.read()


setup(name='connectpyse',
      version='0.4.2',
      description='A ConnectWise API tool for the rest of us.',
      long_description=readme(),
      classifiers=[
          'Development Status :: 3 - Alpha',
          'License :: OSI Approved :: MIT License',
          'Programming Language :: Python :: 3.5'
      ],
      keywords='connectwise rest api',
      url='https://github.com/markciecior/ConnectPyse',
      author='Joshua M Smith (original), Mark Ciecior (forked)',
      author_email='saether@gmail.com (original), mark@markciecior.com (forked)',
      license='MIT',
      packages=['.',
                'connectpyse.expense',
                'connectpyse.finance',
                'connectpyse.marketing',
                'connectpyse.procurement',
                'connectpyse.project',
                'connectpyse.sales',
                'connectpyse.schedule',
                'connectpyse.service',
                'connectpyse.system',
                'connectpyse.time'],
      py_modules = ['connectpyse.cw_controller',
                    'connectpyse.config',
                    'connectpyse.cw_model',
                    'connectpyse.restapi'],
      install_requires=[
          'requests'
      ],
      include_package_data=True,
      zip_safe=False)



from ..cw_controller import CWController
# Class for /service/tickets/{id}/notes
from connectpyse.service import ticket_note


class TicketNotesAPI(CWController):
    def __init__(self, ticket_id):
        self.module_url = 'service'
        self.module = 'tickets/{}/notes'.format(ticket_id)
        self._class = ticket_note.TicketNote
        super().__init__()  # instance gets passed to parent object

    def get_ticket_notes(self):
        return super()._get()

    def create_ticket_note(self, a_ticket_note):
        return super()._create(a_ticket_note)

    def get_ticket_notes_count(self):
        return super()._get_count()

    def get_ticket_note_by_id(self, ticket_note_id):
        return super()._get_by_id(ticket_note_id)

    def delete_ticket_note_by_id(self, ticket_note_id):
        super()._delete_by_id(ticket_note_id)

    def replace_ticket_note(self, ticket_note_id):
        pass

    def update_ticket_note(self, ticket_id, key, value):
        return super()._update(ticket_id, key, value)

    def merge_ticket_note(self, a_ticket_note, target_ticket_note_id):
        # return super()._merge(a_ticket_note, target_ticket_note_id)
        pass

# get
# create
# count
# search
# getbyid
# delete
# replace
# update
# activities
# count
# timeentries
# count
# scheduleentries
# count
# documents
# count
# products
# count
# configurations
# createconfigurations
# count
# betbyid
# delete



from ..cw_model import CWModel


class ExpenseEntry(CWModel):

    def __init__(self, json_dict=None):
        self.id = None  # (Integer)
        self.company = None  # **(CompanyReference)
        self.chargeToId = None  # (Integer)
        self.chargeToType = None  # **(Enum)
        self.type = None  # *(ExpenseTypeReference)
        self.member = None  # (MemberReference)
        self.paymentMethod = None  # (PaymentMethodReference)
        self.classification = None  # (ClassificationReference)
        self.amount = None  # *(Number)
        self.billableOption = None  # *(Enum)
        self.date = None  # *(String)
        self.locationId = None  # (Integer)
        self.businessUnitId = None  # (Integer)
        self.notes = None  # (String)
        self.agreement = None  # (AgreementReference)
        self.invoiceAmount = None  # (Number)
        self.taxes = None  # (ExpenseTax[])
        self.invoice = None  # (InvoiceReference)
        self._info = None  # (Metadata)

        # initialize object with json dict
        super().__init__(json_dict)



from ..cw_controller import CWController
# Class for /service/boards
from . import board


class BoardsAPI(CWController):
    def __init__(self):
        self.module_url = 'service'
        self.module = 'boards'
        self._class = board.Board
        super().__init__()  # instance gets passed to parent object

    def get_boards(self):
        return super()._get()

    def create_board(self, a_entry):
        return super()._create(a_entry)

    def get_boards_count(self):
        return super()._get_count()

    def get_board_by_id(self, entry_id):
        return super()._get_by_id(entry_id)

    def delete_board_by_id(self, entry_id):
        super()._delete_by_id(entry_id)

    def replace_board(self, entry_id):
        pass

    def update_board(self, entry_id, key, value):
        return super()._update(entry_id, key, value)

    def merge_board(self, a_entry, target_entry_id):
        # return super()._merge(a_board, target_board_id)
        pass



from ..cw_model import CWModel


class Batch(CWModel):

    def __init__(self, json_dict=None):
        self.id = None  # (String)
        self.responses = None  # (EndpointResponse[])

        # initialize object with json dict
        super().__init__(json_dict)



from ..cw_model import CWModel


class Callback(CWModel):

    def __init__(self, json_dict=None):
        self.id = None  # (Integer)
        self.description = None  # (String(100))
        self.url = None  # *(String)
        self.objectId = None  # *(Integer)
        self.type = None  # *(String)
        self.level = None  # *(String)
        self.memberId = None  # (Integer)
        self.inactiveFlag = None  # (Boolean)
        self._info = None  # (Metadata)

        # initialize object with json dict
        super().__init__(json_dict)



from os import path
import json
import base64
import sys


sys.path.insert(0, path.dirname(path.abspath(__file__)))

api_file = 'my_api.json'
_api_file = path.join(path.dirname(path.abspath(__file__)), api_file)


with open(_api_file) as fin:
    cw_api_settings = json.load(fin)

API_URL = cw_api_settings['API_URL']
_cid = cw_api_settings['COMPANYID']
_pubk = cw_api_settings['PUBLICKEY']
_privk = cw_api_settings['PRIVATEKEY']
_authtoken = cw_api_settings['AUTHTOKEN']

if (_authtoken is not None) and (_authtoken != "xxxxx"):
    basic_auth = _authtoken

else:

    basic_auth = base64.b64encode("{}+{}:{}".format(_cid, _pubk, _privk).encode('utf-8'))
    basic_auth = {'Authorization': 'Basic {}'.format(str(basic_auth, 'utf-8'))}



from ..cw_model import CWModel


class PurchaseOrderLineItem(CWModel):

    def __init__(self, json_dict=None):
        # With: id(Integer)
        # Search: ^(\s*)(\*?)([_a-z0-9]*)\s?(\(.*$)
        # Replace: $1self.$3 = None  # $2$4
        # To get: self.id = None  # (Integer)
        self.id = None  # (Integer)
        self.backorderedFlag = None  # (Boolean)
        self.canceledBy = None  # (String)
        self.canceledFlag = None  # (Boolean)
        self.canceledReason = None  # (String(100))
        self.closedFlag = None  # (Boolean)
        self.dateCanceled = None  # (String)
        self.dateCanceledUtc = None  # (String)
        self.description = None  # *(String(6000))
        self.displayInternalNotesFlag = None  # (Boolean)
        self.expectedShipDate = None  # (String)
        self.internalNotes = None  # (String(1000))
        self.lineNumber = None  # *(Integer)
        self.packingSlip = None  # (String(30))
        self.product = None  # *(IvItemReference)
        self.purchaseOrderId = None  # (Integer)
        self.quantity = None  # *(Number)
        self.receivedQuantity = None  # (Integer)
        self.serialNumbers = None  # (String(100))
        self.shipDate = None  # (String)
        self.shipmentMethod = None  # (ShipmentMethodReference)
        self.tax = None  # (Number)
        self.trackingNumber = None  # (String(50))
        self.unitCost = None  # (Number)
        self.unitOfMeasure = None  # *(UnitOfMeasureReference)
        self.vendorOrderNumber = None  # (String(50))
        self.warehouse = None  # (WarehouseReference)
        self.warehouseBin = None  # (WarehouseBinReference)
        self._info = None  # (Metadata)

        # initialize object with json dict
        super().__init__(json_dict)
